module Typecheck (typeCheckExp, typeCheckStm, typeCheckVar) where 

import Abssyntax
import Control.Monad.State
import Data.Map as Map
import Prelude hiding (lookup)


type Types = Map Ident Type

inList :: Eq a => a -> [a] -> Bool
inList _ [] = False
inList a (x:xs) = if a == x then True else inList a xs

typeCheckExp :: (MonadState Types m) => Exp -> m Type
typeCheckExp (BiOpExp e o e') = do
	t1 <- typeCheckExp e
	t2 <- typeCheckExp e'
	if not(t1 == t2) then fail "" 
		else case inList o [Eq,NEq] of
			True -> return TBool
			False -> if not(t1 == TInt) then fail "" 
				else case inList o [Plus,Minus,Times,Div] of
					True -> return TInt
					False -> return TBool
typeCheckExp (EVar i) = typeCheckVar i
typeCheckExp (EAss i e) = do
	a <- typeCheckVar i
	b <- typeCheckExp e
	if a == b then return a else fail "FEL!"
typeCheckExp (EInt i) = return TInt
typeCheckExp (EBool b) = return TBool
typeCheckExp EReadI = return TInt
typeCheckExp EReadB = return TBool
typeCheckExp (ExpT t e) = do
	t2 <- typeCheckExp e
	if t == t2 then return t else fail "FEL!"
typeCheckExp EDefault = return NoType
typeCheckExp (EPost i op) = do
	TInt <- typeCheckVar i
	return TInt
typeCheckExp (ENeg e) = do
	TInt <- typeCheckExp e
	return TInt


typeCheckVar :: (MonadState Types m) => Ident -> m Type 
typeCheckVar i = do
	e <- get
	lookup i e

typeCheckStm :: (MonadState Types m) => Stm -> m Type 
typeCheckStm SNoop = return NoType
typeCheckStm (SExp e) = do 
	typeCheckExp e
	return NoType
typeCheckStm (SBlock ss) = do 
	mapM typeCheckStm ss
	return NoType
typeCheckStm (SIf e s s') = do
	TBool <- typeCheckExp e
	NoType <- typeCheckStm s
	NoType <- typeCheckStm s
	return NoType
typeCheckStm (SWhile e s) = do
	TBool <- typeCheckExp e
	NoType <- typeCheckStm s
	return NoType
typeCheckStm (SDecl t i e) = do
	t2 <- typeCheckExp e
	if t == t2 || t2 == NoType then do
		m <- get
		put (insert i t m)
		return NoType
		else fail $ "Illegal to assign an expression of type "++show t2++" to variable "++show i++" of type "++show t
typeCheckStm (SPrint e) = do
	typeCheckExp e
	return NoType

